import { 
  csv,
  nest,
  scaleOrdinal,
  schemeCategory10,
} from 'd3';

import { colorLegend } from './colorLegend';
import { loadAndProcessData } from './loadAndProcessData';

const svg = d3.select('svg');

const width = +svg.attr('width'),
      height = +svg.attr('height');

const render = data => {

  const chartTitle = 'World’s population over time',
  			xValue = d => d.year,
        xAxisLabel = 'Time',
        yValue = d => d.population,
        yAxisLabel = 'Temperature',
        colorValue = d => d.name,
        circleRadius = 3,
        margin = {top: 60, right: 350, bottom: 60, left: 90},
        innerWidth = width - margin.left - margin.right,
        innerHeight = height - margin.top - margin.bottom;
  
  const xScale = d3.scaleTime()
  	.domain(d3.extent(data, xValue))
  	.range([0, innerWidth])
  	// .nice();
  
  const yScale = d3.scaleLinear()
  	.domain(d3.extent(data, yValue).reverse())
  	.range([0, innerHeight])
  	.nice();
  
  const colorScale = scaleOrdinal(schemeCategory10);
  
  const g = svg.append('g')
  	.attr('transform', `translate(${margin.left}, ${margin.top})`);
  
  const yAxis = d3.axisLeft(yScale)
  	.tickSize(-innerWidth)
  	.tickPadding(10);
  
  const yAxisG = g.append('g')
  	.attr('class', 'y-axis')
  	.call(yAxis);
  
  yAxisG.selectAll('.domain')
  		.remove();
  
  yAxisG.append('text')
  	.text(yAxisLabel)
  	.attr('fill', '#999')
  	.attr('x', -innerHeight/2)
  	.attr('y', -60)
  	.attr('transform', `rotate(-90)`)
  	.attr('text-anchor', 'middle')
  
  
  const xAxis = d3.axisBottom(xScale)
  	.tickSize(-innerHeight)
  	.tickPadding(10);
  
  const xAxisG = g.append('g')
  	.attr('class', 'x-axis')
  	.attr('transform', `translate(0, ${innerHeight})`)
  	.call(xAxis)
  	
	xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
  	.text(xAxisLabel)
  	.attr('fill', '#999')
  	.attr('x', innerWidth/2)
  	.attr('y', 50);
  
  
  const lineGenerator = d3.line()
  	.x(d => xScale(xValue(d)) )
  	.y(d => yScale(yValue(d)) )
  	.curve(d3.curveBasis);
  
  const nested = nest()
  	.key(d => d.name)
  	.entries(data);
  
  console.log(nested);
  
  const lastYValue = d => yValue(d.values[ d.values.length-1 ]);
  
  colorScale.domain(
    nested.sort((a,b) => d3.descending(lastYValue(a), lastYValue(b)))
    	.map(d => d.key)
  );
  
  
  g.selectAll('.line-path').data(nested)
  	.enter().append('path')
      .attr('class', 'line-path')
      .attr('d', d => lineGenerator(d.values))
  		.attr('stroke', d => colorScale(d.key));
  
  g.append('text')
  	.attr('class', 'h1')
  	.attr('y', -30)
  	.text(chartTitle)
  	.style('font-weight', 'bold')
  
  svg.append('g')
	.attr('transform', `translate(${width - margin.right + 20}, ${margin.top})`)
	.call(colorLegend, {
    circleRadius: 8,
  	circleSpacing: 30,
    colorScale,
  	textOffset: 15,
	});
  
};

loadAndProcessData()
	.then(render