import { 
  csv, 
  json, 
  range,
  timeParse
} from 'd3';

const allCaps = str => str === str.toUpperCase();
const isRegion = c => allCaps(c) && c !== 'WORLD';

const parseYear = timeParse('%Y');

const melt = (data, minYear, maxYear) => {
    
    const years = range(minYear, maxYear+1);
    const meltedData = [];
    
    
    data.forEach(d => {
      const name = d["Region, subregion, country or area *"]
      	.replace(/\(.+\)/, '');
      
      if (!isRegion(name))
        return;
        
      years.forEach(year => {
        const population = +d[year].replace(/ /g,'');
        const row = {
          year: parseYear(year),
          name,
          population
        };
        meltedData.push(row);
      })
    });
  
  return meltedData;
}

export const loadAndProcessData = () =>

  Promise.all([
      csv('https://vizhub.com/arieltonglet/datasets/un-population-variant.csv'),
      csv('https://vizhub.com/arieltonglet/datasets/wpp2019_pop_f01_1_total_population_both_sexes.csv')
  	])
	.then(([unData, unLegacyData]) => {
    
    console.log(unLegacyData.columns)

    const meltedUnData = melt(unData, 2020, 2100);
    const meltedUnLegacyData = melt(unLegacyData, 1950, 2020);
    
    return meltedUnLegacyData.concat(meltedUnData);
    
});